@@ -17,9 +17,31 @@ from importlib.metadata import version
 
 # Set up globals
 __version__ = version('xwordlist')
-exec_pieces = os.path.splitext(os.path.basename(__file__))
-exec_name = exec_pieces[0]
-config_name = os.path.join(os.path.dirname(os.path.abspath(__file__)), f'{exec_name}.conf')
+
+FILE = {
+    'name': __name__,
+    'conf': f'{__name__}.conf',
+    'exec_path': os.path.dirname(os.path.abspath(__file__)),
+    'user_path': os.path.join(pathlib.Path.home(), __name__, '')
+}
+
+REPO = {
+    'home': 'https://github.com/aanker/xwordlist/',
+    'conf': 'https://github.com/aanker/xwordlist/blob/main/xwordlist.conf',
+}
+
+CONFIG_EXEC = os.path.join(FILE['exec_path'], FILE['conf'])
+CONFIG_HOME = os.path.join(FILE['user_path'], FILE['conf'])
+
+DEFAULTS = {
+    'convert': ' ',
+    'strip': 'diacritic',
+    'minimum': 3,
+    'case': 'upper',
+    'dedupe': 'nocase',
+    'webextract': 'text',
+    'alphabetize': 'normal',
+}
 
 GLOBAL_SETTINGS = {
     'urllist_delay': 20,
@@ -28,7 +50,6 @@ GLOBAL_SETTINGS = {
 }
 
 IMPACT_COLOR = GLOBAL_SETTINGS['impact_color']
-
 COLOR_OPTIONS = ['ansiblack', 'ansired', 'ansigreen', 'ansiyellow', 'ansiblue', 'ansimagenta',
                  'ansicyan', 'ansigray', 'ansibrightblack', 'ansibrightred', 'ansibrightgreen',
                  'ansibrightyellow', 'ansibrightblue', 'ansibrightmagenta', 'ansibrightcyan', 'ansiwhite']
@@ -39,16 +60,16 @@ class WordList:
         self.myList = myList
 
     # List transformation options
-    def remove_min(self, numChars):
+    def minimum(self, numChars):
         newList = []
         for line in self.myList:
-            if line and len(line) >= numChars:
+            if line and len(line) >= int(numChars):
                 newList.append(line)
         self.myList = newList
 
-    def strip_nonalpha(self, stripWhat):
+    def strip(self, stripWhat):
         newList = []
-        if stripWhat in ['keepdiacritic', 'diacritic', 'true']:
+        if stripWhat in ['keepdiacritic', 'diacritic']:
             for word in self.myList:
                 if stripWhat != 'keepdiacritic':
                     word = anyascii(word)
@@ -61,7 +82,7 @@ class WordList:
             print_line(print_text, {'stripWhat': stripWhat})
             sys.exit()
 
-    def case_change(self, newCase):
+    def case(self, newCase):
         case_dict = {
             'upper': str.upper,
             'lower': str.lower,
@@ -73,10 +94,10 @@ class WordList:
             print_line(print_text, {'newcase': newCase})
             sys.exit()
 
-    def uniquify(self, dedupeType):
+    def dedupe(self, dedupeType):
         if dedupeType == 'bycase':
             self.myList = list(dict.fromkeys(self.myList))
-        elif dedupeType in ['nocase', 'true']:
+        elif dedupeType == 'nocase':
             newList = []
             newSet = set()
             for line in self.myList:
@@ -90,8 +111,15 @@ class WordList:
             print_line(print_text, {'dedupeType': dedupeType})
             sys.exit()
 
-    def alphabetize(self):
-        self.myList = sorted(self.myList, key=str.casefold)
+    def alphabetize(self, direction):
+        if direction == 'normal':
+            self.myList = sorted(self.myList, key=str.casefold)
+        elif direction == 'reverse':
+            self.myList = sorted(self.myList, key=str.casefold, reverse=True)
+        else:
+            print_text = 'Exiting... incorrect alphabetize option <{color}>{direction}</{color}>'
+            print_line(print_text, {'direction': direction})
+            sys.exit()
 
     # Content parsing options
     def regex(self, regexInput):
@@ -112,8 +140,6 @@ class WordList:
 
     def convert(self, parseChars):
         # First trap for problem where defaults are assumed but not specified
-        if parseChars == 'true':
-            parseChars = ' '
         for char in parseChars:
             newList = []
             for line in self.myList:
@@ -154,7 +180,7 @@ def create_dict(localAttrs):
 
 def extract_from_web(extractWhat, soup, extractURL):
     # A few ways the default option can come in, try that first
-    if extractWhat == 'text' or extractWhat == 'true' or extractWhat is None:
+    if extractWhat == 'text' or extractWhat is None:
         localWords = soup.stripped_strings
     elif extractWhat == 'links':
         localWords = []
@@ -328,19 +354,84 @@ def setup_input(localArgs, otherArgs):
 
     if len(returnWords) == 0:
         print_text = 'No input given, nothing to do (enter <{color}>{exec_name} -h</{color}> for help)'
-        print_line(print_text, {'exec_name': exec_name})
+        print_line(print_text, {'exec_name': FILE['name']})
         sys.exit()
 
     return returnWords
 
 
+def do_config():
+    # Tells user where their config file is
+    if pathlib.Path(CONFIG_EXEC).is_file() and pathlib.Path(CONFIG_HOME).is_file():
+        print_text = 'There are configuration files located at both <{color}>{config_exec}</{color}>'
+        print_text += ' and <{color}>{config_home}</{color}>.\nThe one located in the folder <{color}>{home_path}</{color}>'
+        print_text += ' will take precedence and is the one you should edit.'
+        arg_dict = {
+            'config_home': CONFIG_HOME,
+            'config_exec': CONFIG_EXEC,
+            'home_path': FILE['user_path'],
+        }
+        print_line(print_text, arg_dict)
+        sys.exit()
+
+    elif pathlib.Path(CONFIG_EXEC).is_file():
+        whereFile = CONFIG_EXEC
+    elif pathlib.Path(CONFIG_HOME).is_file():
+        whereFile = CONFIG_HOME
+    else:
+        print_text = 'Cannot find a configuration file.\nYou should download a new version from this'
+        print_text += ' location: <{color}>{repo_conf}</{color}>\nSave it in a directory'
+        print_text += ' located at <{color}>{home_path}</{color}> and edit it there.'
+        arg_dict = {
+            'repo_conf': REPO['conf'],
+            'home_path': FILE['user_path'],
+        }
+        print_line(print_text, arg_dict)
+        sys.exit()
+
+    print_text = 'Your configuration file is located at <{color}>{whereFile}</{color}>'
+    print_line(print_text, {'whereFile': whereFile})
+    sys.exit()
+
+
+def do_word_options(localWords, localArgs):
+    thatOption = ''
+    if localArgs.line2word and localArgs.word2word:
+        print_text = 'Options <{color}>line2word</{color}> and <{color}>word2word</{color}>'
+        print_text += ' are mutually exclusive, please use either but not both'
+        print_line(print_text)
+        sys.exit()
+    elif localArgs.line2word:
+        # If line2word we first do a convert
+        localWords.convert(DEFAULTS['convert'])
+        thatOption = 'line2word'
+    # Now run through all the other options
+    optionList = ['strip', 'minimum', 'case', 'dedupe', 'alphabetize']
+    for option in optionList:
+        getattr(localWords, option)(DEFAULTS[option])
+
+    if thatOption == '':
+        thatOption = 'word2word'
+    return localWords, thatOption
+
+
+def do_ignore(whichOption, thatOption):
+    print_text = 'Option <{color}>{whichOption}</{color}> ignored due to use of <{color}>{thatOption}</{color}>'
+    arg_dict = {
+        'whichOption': whichOption,
+        'thatOption': thatOption,
+    }
+    print_line(print_text, arg_dict)
+
+
 def main():
     # First set up configargparse
-    parser = configargparse.ArgumentParser(default_config_files=[config_name],
+    parser = configargparse.ArgumentParser(default_config_files=[CONFIG_EXEC, CONFIG_HOME],
                                            description='Crossword puzzle word list builder')
 
     # Meta options
-    parser.add_argument('-v', '--version', action='version', version=f'{exec_name} {__version__}')
+    parser.add_argument('-v', '--version', action='version', version=f"{FILE['name']} {__version__}")
+    parser.add_argument('--config', action='store_true', help='locate your config file and quit')
 
     # Input and output options
     parser.add_argument('-i', '--input', type=pathlib.Path, help='Input text file')
@@ -356,40 +447,60 @@ def main():
                       using tag=term syntax (e.g., id=main_content or class=lyrics).'
     parser.add_argument('--container', nargs=1, help=container_help)
     webextract_help = 'Specify whether to extract text, links or specific tags from web inputs'
-    parser.add_argument('--webextract', nargs='?', default='text', help=webextract_help)
-    convert_help = 'Convert a block of text to a word list. Default delimiter is a space but acccepts\
-                    any number of characters in quotes (e.g., --convert " ;," will separate words delimited\
-                    by a space, comma or semicolon). Be careful with back slashes acting as an escape character'
-    parser.add_argument('--convert', nargs='?', const=' ', help=convert_help)
+    parser.add_argument('--webextract', nargs='?', default=DEFAULTS['webextract'], help=webextract_help)
     parser.add_argument('--regex', nargs=1, help='Parse text based on regex')
 
     # List transformation options
-    parser.add_argument('-a', '--alphabetize', action='store_true', help='Alphabetize the list')
-    case_help = ' {none (default) | lower | upper} Change the case of words in the list'
-    parser.add_argument('--case', nargs='?', const='none', help=case_help)
+    line2word_help = 'Convert each line of text into a word by stripping out spaces and other non-alphabetic\
+                      characters, dedupe the list, alphabetize and capitalize every word. Remove all words\
+                      below the minimum word size. This is the equivalent of running xwordlist with each of those\
+                      options set and saves having to enter each one individually. See the help documentation to\
+                      better understand how this works.'
+    parser.add_argument('--line2word', action='store_true', help=line2word_help)
+    word2word_help = 'Convert blocks of text into individual words delimited by spaces, strip out non-alphabetic\
+                      characters, dedupe the list, alphabetize and capitalize every word. Remove all words\
+                      below the minimum word size. This is the equivalent of running xwordlist with each of those\
+                      options set and saves having to enter each one individually. See the help documentation to\
+                      better understand how this works.'
+    parser.add_argument('--word2word', action='store_true', help=word2word_help)
+    convert_help = 'Convert a block of text into individual words, separating words by spaces. See help\
+                    documentation for additional options'
+    parser.add_argument('--convert', nargs='?', const=DEFAULTS['convert'], help=convert_help)
+    alphabetize_help = '{normal (default) | reverse} Alphabetize the list'
+    parser.add_argument('-a', '--alphabetize', nargs='?', const=DEFAULTS['alphabetize'], help=alphabetize_help)
+    case_help = ' {upper (default) | lower | none} Change the case of words in the list'
+    parser.add_argument('--case', nargs='?', const=DEFAULTS['case'], help=case_help)
     dedupe_help = '{nocase (default) | bycase} Remove duplicates from the word list. By default, ignores\
                    case: "apple" and "APPLE" are the same word and the first instance found is kept.\
                    Use --dedupe bycase to treat each as a different word'
-    parser.add_argument('-d', '--dedupe', nargs='?', const='nocase', help=dedupe_help)
-    min_ltrs = 3
-    minimum_help = f'Set minimum number of letters in a word (if not specified, default is {min_ltrs})'
-    parser.add_argument('-m', '--minimum', nargs='?', type=int, const=min_ltrs, help=minimum_help)
+    parser.add_argument('-d', '--dedupe', nargs='?', const=DEFAULTS['dedupe'], help=dedupe_help)
+    minimum_help = f"Set minimum number of letters in a word (if not specified, default is {DEFAULTS['minimum']})"
+    parser.add_argument('-m', '--minimum', nargs='?', const=DEFAULTS['minimum'], help=minimum_help)
     strip_help = '{diacritic (default) | keepdiacritic} Remove non-alphabetic characters (including spaces).\
                   By default, converts diacritical marks into English letters  (e.g., Renée becomes Renee).\
                   Use --strip keepdiacritic to leave diacriticals in.'
-    parser.add_argument('-s', '--strip', nargs='?', const='diacritic', help=strip_help)
+    parser.add_argument('-s', '--strip', nargs='?', const=DEFAULTS['strip'], help=strip_help)
 
     args = parser.parse_known_args()
     confArgs = args[0]
-    envArgs = create_dict(args[1])
-    # print(confArgs)
-    # sys.exit()
+
+    # Check to see if we've found any configuration data
+    envArgs = create_dict(args[1]) if len(args[1]) > 0 else {}
+
+    # Update items with defaults that come through as 'true'
+    for arg in DEFAULTS:
+        if getattr(confArgs, arg) == 'true':
+            setattr(confArgs, arg, DEFAULTS[arg])
 
     # See if conf file contains an impact color
     if 'impact_color' in envArgs and f"ansi{envArgs['impact_color']}" in COLOR_OPTIONS:
         global IMPACT_COLOR
         IMPACT_COLOR = f"ansi{envArgs['impact_color']}"
 
+    # See if the user selected --config, in which case we should handle that immediately
+    if confArgs.config:
+        do_config()
+
     # See if a default directory was specified and rewrite inputs and outputs as necessary
     if confArgs.directory is not None:
         if pathlib.Path(confArgs.directory).is_dir():
@@ -401,6 +512,7 @@ def main():
             print_line(print_text, {'directory': confArgs.directory})
             sys.exit()
 
+    # Set up output file to make sure it doesn't exist then grab all inputs (including web parsing)
     outputFile = setup_output(confArgs, envArgs)
     inputWords = WordList(setup_input(confArgs, envArgs))
 
@@ -413,24 +525,20 @@ def main():
             sys.exit()
         inputWords.regex(confArgs.regex[0])
 
-    if confArgs.convert is not None:
-        inputWords.convert(confArgs.convert)
+    # First figure out if they set either combination option
+    if confArgs.line2word or confArgs.word2word:
+        inputWords, thatOption = do_word_options(inputWords, confArgs)
+    else:
+        thatOption = False
 
-    if confArgs.strip is not None:
-        inputWords.strip_nonalpha(confArgs.strip)
-
-    # Do any text transforms
-    if confArgs.minimum is not None:
-        inputWords.remove_min(confArgs.minimum)
-
-    if confArgs.case is not None:
-        inputWords.case_change(confArgs.case)
-
-    if confArgs.dedupe is not None:
-        inputWords.uniquify(confArgs.dedupe)
-
-    if confArgs.alphabetize:
-        inputWords.alphabetize()
+    # Now run through remaining options
+    # We also trap for the case where the user had one of the line2word or word2word options
+    # but also uses one of the duplicative other options (we ignore the duplicative one)
+    optionList = ['convert', 'strip', 'minimum', 'case', 'dedupe', 'alphabetize']
+    for option in optionList:
+        confOption = getattr(confArgs, option)
+        if confOption is not None:
+            do_ignore(option, thatOption) if thatOption else getattr(inputWords, option)(confOption)
 
     # Now save the file
     save_output(outputFile, inputWords)
